﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Core.Attributes
{
    /// <summary>
    /// 数据权限特性，仅允许读取指定用户/租户的数据
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true)]
    public class DataPermissionAttribute : Attribute
    {
        /// <summary>
        /// 用户所有者（仅当前用户）
        /// </summary>
        public virtual bool UserOwner { get; set; }
        /// <summary>
        /// 租户所有者（仅当前租户），暂不使用
        /// </summary>
        public virtual bool TenantOwner { get; set; }

        /// <summary>
        /// 当前用户Id，框架自动填充
        /// </summary>
        public virtual string UserId { get; set; }
        /// <summary>
        /// 当前租户Id，框架自动填充
        /// </summary>
        public virtual string TenantId { get; set; }

    }
}
