﻿using SixLabors.ImageSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace Core.Entitys
{
    /// <summary>
    /// 系统菜单
    /// </summary>
    public class SysMenu : EntityBase
    {

        /// <summary>
        /// 菜单类型（1目录 2菜单 3按钮）
        /// </summary>
        [SugarColumn(ColumnDescription = "菜单类型（1目录 2菜单 3按钮）")]
        public MenuTypeEnum type {  get; set; }

        /// <summary>
        /// 路由名称
        /// </summary>
        [SugarColumn(ColumnDescription = "路由名称")]
        public string? Name {  get; set; } = string.Empty;

        /// <summary>
        /// 路由地址
        /// </summary>
        [SugarColumn(ColumnDescription = "路由地址")]
        public string? Path {  get; set; } = string.Empty;

        /// <summary>
        /// 权限标识
        /// </summary>
        [SugarColumn(ColumnDescription = "权限标识")]
        public string? Permission { get; set; } = string.Empty;

        /// <summary>
        /// 排序
        /// </summary>
        [SugarColumn(ColumnDescription = "OrderNo")]
        public int OrderNo {  get; set; }

        /// <summary>
        /// 父Id
        /// </summary>
        [SugarColumn(ColumnDescription = "父Id")]
        [JsonConverter(typeof(ValueToStringConverter))]
        public long ParentId { get; set; }

        /// <summary>
        /// 子菜单
        /// </summary>
        [SugarColumn(IsIgnore = true)]
        public List<SysMenu> Children { get; set; }


        /// <summary>
        /// 组件路径
        /// </summary>
        [SugarColumn(ColumnDescription = "组件路径")]
        public string? Component { get; set; } = string.Empty;

        /// <summary>
        /// 布局组件路径
        /// </summary>
        [SugarColumn(ColumnDescription = "布局组件路径")]
        public string? LayoutComponent { get; set; }

        /// <summary>
        /// 菜单名称
        /// </summary>
        [SugarColumn(ColumnDescription = "菜单名称")]
        public string Title { get; set; } = string.Empty;

        /// <summary>
        /// 图标，
        /// 
        /// 支持单个svg，示例：icon-a1
        /// 
        /// 支持多个svg，示例：icon-a1,icon-a2,icon-a3【正常,覆盖,激活】
        /// </summary>
        [SugarColumn(ColumnDescription = "图标，支持单个svg，示例：icon-a1; 支持多个svg，示例：icon-a1,icon-a2,icon-a3【正常,覆盖,激活】")]
        public string Icon { get; set; } = string.Empty;

        /// <summary>
        /// 自定义布局
        /// </summary>
        [SugarColumn(ColumnDescription = "自定义布局")]
        public bool HasLayout { get; set; } = false;

        /// <summary>
        /// 显示侧边栏
        /// </summary>
        [SugarColumn(ColumnDescription = "显示侧边栏")]
        public bool HasSider { get; set; } = true;

        /// <summary>
        /// 是否显示面包屑
        /// </summary>
        [SugarColumn(ColumnDescription = "是否显示面包屑")]
        public bool IsBreadcrumb { get; set; } = true;

        /// <summary>
        /// 是否内嵌
        /// </summary>
        [SugarColumn(ColumnDescription = "是否内嵌")]
        public bool IsIframe { get; set; } = false;

        /// <summary>
        /// 是否外链
        /// </summary>
        [SugarColumn(ColumnDescription = "是否外链")]
        public bool IsLink { get; set; } = false;

        /// <summary>
        /// 外链链接，支持 flags（标志位）参数
        /// </summary>
        [SugarColumn(ColumnDescription = "外链链接，支持 flags（标志位）参数")]
        public string OutLink { get; set; } = string.Empty;

        /// <summary>
        /// 外链链接目标('_blank' | '_self')
        /// </summary>
        [SugarColumn(ColumnDescription = "外链链接目标('_blank' | '_self')")]
        public string? OutLinkTarget { get; set; } = "_blank";

        /// <summary>
        /// 是否隐藏
        /// </summary>
        [SugarColumn(ColumnDescription = "是否隐藏")]
        public bool IsHide { get; set; } = false;

        /// <summary>
        /// 是否缓存
        /// </summary>
        [SugarColumn(ColumnDescription = "是否缓存")]
        public bool IsKeepAlive { get; set; } = false;

        /// <summary>
        /// 是否固定
        /// </summary>
        [SugarColumn(ColumnDescription = "是否固定")]
        public bool IsAffix { get; set; } = false;
    }
}
