﻿using Core.General;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Core.Entitys
{
    /// <summary>
    /// 系统用户表
    /// </summary>
    public class SysUser : EntityBaseData
    {
        /// <summary>
        /// 账号类型
        /// </summary>
        [SugarColumn(ColumnDescription = "账号类型")]
        public virtual AccountTypeEnum AccountType { get; set; } = AccountTypeEnum.NormalUser;

        /// <summary>
        /// 昵称
        /// </summary>
        [SugarColumn(ColumnDescription = "昵称")]
        [MaxLength(32)]
        public virtual string NickName { get; set; } = string.Empty;

        /// <summary>
        /// 头像
        /// </summary>
        [SugarColumn(ColumnDescription = "头像")]
        [MaxLength(512)]
        public virtual string Avatar { get; set; } = string.Empty;

        /// <summary>
        /// 性别
        /// </summary>
        [SugarColumn(ColumnDescription = "性别")]
        public virtual SexEnum Sex { get; set; } = SexEnum.Unknown;

        /// <summary>
        /// 手机号码
        /// </summary>
        [SugarColumn(ColumnDescription = "手机号码")]
        [MaxLength(11), Phone]
        public virtual string Phone { get; set; } = string.Empty;

        /// <summary>
        /// 备注
        /// </summary>
        [SugarColumn(ColumnDescription = "备注")]
        public virtual string Remark { get; set; } = string.Empty;

        /// <summary>
        /// 状态
        /// </summary>
        [SugarColumn(ColumnDescription = "状态")]
        public virtual StatusEnum Status { get; set; } = StatusEnum.Enable;

        /// <summary>
        /// 直属机构Id
        /// </summary>
        [SugarColumn(ColumnDescription = "直属机构Id")]
        [JsonConverter(typeof(ValueToStringConverter))]
        public long OrgId { get; set; }

        /// <summary>
        /// 直属机构
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(OrgId))]
        public SysOrg SysOrg { get; set; }

        /// <summary>
        /// 直属主管Id
        /// </summary>
        [SugarColumn(ColumnDescription = "直属主管Id")]
        [JsonConverter(typeof(ValueToStringConverter))]
        public long? ManagerUserId { get; set; }

        /// <summary>
        /// 直属主管
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(ManagerUserId))]
        public SysUser ManagerUser { get; set; }
    }
}
