﻿//using Core;
//using Core.Attributes;
//using Microsoft.AspNetCore.Http;
//using Microsoft.EntityFrameworkCore;
//using Minio.DataModel;
//using System;
//using System.Collections.Generic;
//using System.Collections.Specialized;
//using System.Diagnostics.CodeAnalysis;
//using System.Linq;
//using System.Linq.Expressions;
//using System.Reflection;
//using System.Runtime.CompilerServices;
//using System.Text;
//using System.Text.Json.Serialization;
//using System.Threading.Tasks;
//using System.Xml.Linq;

//namespace Core.Extensions
//{
//    /// <summary>
//    /// 数据库通用查询参数
//    /// </summary>
//    public sealed class DatabaseQueryParameter<Entity>
//    {
//        private readonly IHttpContextAccessor accessor;
//        /// <summary>
//        /// 
//        /// </summary>
//        /// <param name="accessor"></param>
//        public DatabaseQueryParameter(IHttpContextAccessor accessor)
//        {
//            this.accessor = accessor;
//        }
//        /// <summary>
//        /// 页码
//        /// </summary>
//        public int Page { get; private set; }
//        /// <summary>
//        /// 页数量
//        /// </summary>
//        public int PageSize { get; private set; }
//        /// <summary>
//        /// 排序字段
//        /// </summary>
//        public List<Tuple<string, bool>> OrderFields { get; } = new List<Tuple<string, bool>>();
//        /// <summary>
//        /// 额外的查询条件
//        /// </summary>
//        public List<Expression<Func<Entity, bool>>> Predicates { get; } = new List<Expression<Func<Entity, bool>>>();

//        /// <summary>
//        /// 设置页码，当页面为0时代表查询所有数据
//        /// </summary>
//        /// <param name="page"></param>
//        public void SetPage(int page)
//        {
//            Page = page;
//        }

//        /// <summary>
//        /// 设置页数量
//        /// </summary>
//        /// <param name="pageSize"></param>
//        public void SetPageSize(int pageSize)
//        {
//            PageSize = pageSize;
//        }
//        /// <summary>
//        /// 添加排序字段
//        /// </summary>
//        /// <param name="property"></param>
//        /// <param name="ascending"></param>
//        public void AddOrderField(string property, bool ascending)
//        {
//            if (!string.IsNullOrWhiteSpace(property))
//            {
//                property.Split(",", StringSplitOptions.TrimEntries).ToList().ForEach(property2 => OrderFields.Add(Tuple.Create(property2, ascending)));
//            }
//        }
//        /// <summary>
//        /// 添加用户数据权限
//        /// </summary>
//        /// <returns></returns>
//        public void AddUserDataPermission()
//        {
//            var userId = accessor.HttpContext?.User.Claims.GetUserId();
//            Predicates.Add(o => (o as IPermissionEntity).UserId == userId);
//        }
//    }
//    /// <summary>
//    /// 数组结果
//    /// </summary>
//    /// <typeparam name="T"></typeparam>
//    public sealed class ListResult<T>
//    {
//        internal ListResult(List<T> result, int total)
//        {
//            this.Records = result;
//            this.Total = total;
//            this.PageSize = result.Count;
//        }

//        internal ListResult(List<T> result, int total, int page, int pageSize)
//        {
//            this.Records = result;
//            this.Total = total;
//            this.Current = page;
//            this.PageSize = pageSize;
//        }
//        /// <summary>
//        /// 查询结果数组
//        /// </summary>
//        public List<T> Records { get; }
//        /// <summary>
//        /// 满足查询条件总数
//        /// </summary>
//        public int Total { get; }
//        /// <summary>
//        /// 页码
//        /// </summary>
//        public int Current { get; } = 1;
//        /// <summary>
//        /// 页数量
//        /// </summary>
//        public int PageSize { get; } = 10;

//        /// <summary>
//        /// 是否有下一页
//        /// </summary>
//        public bool Next { get => Current < Pages; }
//        /// <summary>
//        /// 是否有上一页
//        /// </summary>
//        public bool Prev { get => Current > 1; }
//        /// <summary>
//        /// 总页数
//        /// </summary>
//        public int Pages
//        {
//            get
//            {
//                if (PageSize <= 0) return 0;
//                return Total / PageSize + (Total % PageSize == 0 ? 0 : 1);
//            }
//        }
//    }

//    /// <summary>
//    /// 数据库访问对象拓展
//    /// </summary>
//    public static class DatabaseExtensions
//    {
//        /// <summary>
//        /// 附加PermissionEntity（自动设置数据权限）
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <param name="entitys"></param>
//        static void AdditionalPermission<Entity>(this IDatabase database, params Entity[] entitys)
//        {
//            if (entitys.Length == 0) return;
//            IHttpContextAccessor accessor = database.Accessor;
//            // 未获取到请求上下文
//            if (accessor == null || accessor.HttpContext == null || accessor.HttpContext.User == null) { return; }
//            var claims = accessor.HttpContext.User.Claims;
//            var userId = claims.GetUserId(); // 获取用户Id
//            var tenantId = claims.GetTenantId(); // 获取租户Id
//            foreach (Entity entity in entitys)
//            {
//                // 如果数据实体是权限实体时，如果属性字段无值或空字符串是，读取当前请求上下文，设置权限Id
//                if (entity is IPermissionEntity e)
//                {
//                    if (!string.IsNullOrEmpty(userId) && string.IsNullOrEmpty(e.UserId)) e.UserId = userId;
//                    if (!string.IsNullOrEmpty(tenantId) && string.IsNullOrEmpty(e.TenantId)) e.TenantId = tenantId;
//                }
//            }
//        }

//        #region Add

//        public static Entity[] Add<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            // 附加权限数据
//            database.AdditionalPermission(entitys);

//            var context = (DbContext)database;
//            context.AddRange(entitys);
//            context.SaveChanges();
//            return entitys;
//        }

//        public static async Task<Entity[]> AddAsync<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            // 附加权限数据
//            database.AdditionalPermission(entitys);

//            var context = (DbContext)database;
//            context.AddRange(entitys);
//            await context.SaveChangesAsync();
//            return entitys;
//        }

//        #endregion

//        #region Delete


//        public static int Delete<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            var context = (DbContext)database;
//            context.RemoveRange(entitys);
//            return context.SaveChanges();
//        }

//        /// <summary>
//        /// 按条件删除数据
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <param name="predicates"></param>
//        /// <returns></returns>
//        public static int DeleteByPredicate<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            if (predicates.Length == 0) return 0;

//            var entitys = database.GetObjectList(predicates);
//            var context = (DbContext)database;
//            context.RemoveRange(entitys);
//            return context.SaveChanges();
//        }


//        public static async Task<int> DeleteAsync<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            var context = (DbContext)database;
//            context.RemoveRange(entitys);
//            return await context.SaveChangesAsync();
//        }

//        /// <summary>
//        /// 按条件删除数据，注意：不输入条件时将删除所有数据
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <param name="predicates"></param>
//        /// <returns></returns>
//        public static async Task<int> DeleteByPredicateAsync<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            if (predicates.Length == 0) return 0;

//            var entitys = database.GetObjectList(predicates);
//            var context = (DbContext)database;
//            context.RemoveRange(entitys);
//            return await context.SaveChangesAsync();
//        }

//        #endregion

//        #region Update


//        public static Entity[] Update<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            var context = (DbContext)database;
//            context.UpdateRange(entitys);
//            context.SaveChanges();
//            return entitys;
//        }

//        public static async Task<Entity[]> UpdateAsync<Entity>(this IDatabase database, params Entity[] entitys) where Entity : class
//        {
//            var context = (DbContext)database;
//            context.UpdateRange(entitys);
//            await context.SaveChangesAsync();
//            return entitys;
//        }

//        #endregion

//        #region AsQueryable
//        /// <summary>
//        /// 获取Linq查询对象(自定义高级查询)
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <returns></returns>
//        public static IQueryable<Entity> AsQueryable<Entity>(this IDatabase database) where Entity : class
//        {
//            //var ermission = typeof(Entity).GetCustomAttribute<DataPermissionAttribute>();
//            var context = (DbContext)database;
//            return context.Set<Entity>().AsQueryable<Entity>().AutoInclude(database);
//        }

//        /// <summary>
//        /// 获取Linq查询对象(自定义高级查询)
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <returns></returns>
//        public static IQueryable<Entity> AsQueryableAndUserDataPermission<Entity>(this IDatabase database) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            var predicates = new List<Expression<Func<Entity, bool>>>();

//            var userId = database.Accessor.HttpContext?.User.Claims.GetUserId();
//            predicates.Add(o => (o as IPermissionEntity).UserId == userId);

//            // 额外查询条件
//            if (predicates.Count > 0)
//            {
//                foreach (var predicate2 in predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }
//            return queryable;
//        }
//        #endregion

//        #region AsOriginal
//        /// <summary>
//        /// 获取原始对象
//        /// </summary>
//        /// <typeparam name="Entity"></typeparam>
//        /// <param name="database"></param>
//        /// <returns></returns>
//        public static DbContext AsOriginal(this IDatabase database)
//        {
//            return (DbContext)database;
//        }
//        #endregion

//        #region GetObject
//        public static Entity? GetObject<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return queryable.FirstOrDefault();
//        }

//        public static Entity? GetObject<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);

//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            return queryable.FirstOrDefault();
//        }

//        public static async Task<Entity?> GetObjectAsync<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return await queryable.FirstOrDefaultAsync();
//        }

//        public static async Task<Entity?> GetObjectAsync<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);

//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            return await queryable.FirstOrDefaultAsync();
//        }

//        #endregion

//        #region GetObjectById
//        public static Entity? GetObjectById<Entity>(this IDatabase database, params object[] keys) where Entity : class
//        {
//            var context = (DbContext)database;
//            return context.Find<Entity>(keys);
//        }
//        public static async Task<Entity?> GetObjectByIdAsync<Entity>(this IDatabase database, params object[] keys) where Entity : class
//        {
//            var context = (DbContext)database;
//            return await context.FindAsync<Entity>(keys);
//        }

//        #endregion

//        public static int GetCount<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return queryable.Count();
//        }

//        public static int GetCount<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return queryable.Count();
//        }
//        public static async Task<int> GetCountAsync<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return await queryable.CountAsync();
//        }

//        public static async Task<int> GetCountAsync<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);

//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            return await queryable.CountAsync();
//        }

//        #region GetObjectList
//        public static List<Entity> GetObjectList<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return queryable.ToList();
//        }
//        public static async Task<List<Entity>> GetObjectListAsync<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            return await queryable.ToListAsync();
//        }

//        public static List<Entity> GetObjectList<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            // 先排序后limit
//            if (parameter.OrderFields.Count > 0)
//            {
//                foreach (var field in parameter.OrderFields)
//                {
//                    queryable = queryable.Order(field.Item1, field.Item2);
//                }
//            }

//            if (parameter.Page > 0 && parameter.PageSize >= 0)
//            {
//                queryable = queryable.Skip((parameter.Page - 1) * parameter.PageSize).Take(parameter.PageSize);
//            }
//            return queryable.ToList();
//        }

//        public static async Task<List<Entity>> GetObjectListAsync<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);

//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            // 先排序后limit
//            if (parameter.OrderFields.Count > 0)
//            {
//                foreach (var field in parameter.OrderFields)
//                {
//                    queryable = queryable.Order(field.Item1, field.Item2);
//                }
//            }

//            if (parameter.Page > 0 && parameter.PageSize >= 0)
//            {
//                queryable = queryable.Skip((parameter.Page - 1) * parameter.PageSize).Take(parameter.PageSize);
//            }


//            return await queryable.ToListAsync();
//        }

//        #endregion

//        #region GetObjectListAndCount
//        public static ListResult<Entity> GetObjectListAndCount<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            var total = queryable.Count();
//            var result = queryable.ToList();
//            return new ListResult<Entity>(result, total);
//        }
//        public static async Task<ListResult<Entity>> GetObjectListAndCountAsync<Entity>(this IDatabase database, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            var total = await queryable.CountAsync();
//            var result = await queryable.ToListAsync();
//            return new ListResult<Entity>(result, total);
//        }

//        public static ListResult<Entity> GetObjectListAndCount<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);
//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            // 先排序后limit
//            if (parameter.OrderFields.Count > 0)
//            {
//                foreach (var field in parameter.OrderFields)
//                {
//                    queryable = queryable.Order(field.Item1, field.Item2);
//                }
//            }
//            var total = queryable.Count();
//            if (parameter.Page > 0 && parameter.PageSize >= 0)
//            {
//                queryable = queryable.Skip((parameter.Page - 1) * parameter.PageSize).Take(parameter.PageSize);
//            }

//            var result = queryable.ToList();
//            return new ListResult<Entity>(result, total, parameter.Page, parameter.PageSize);
//        }

//        public static async Task<ListResult<Entity>> GetObjectListAndCountAsync<Entity>(this IDatabase database, Action<DatabaseQueryParameter<Entity>> query, params Expression<Func<Entity, bool>>[] predicates) where Entity : class
//        {
//            var queryable = database.AsQueryable<Entity>();
//            foreach (var predicate in predicates)
//                queryable = queryable.Where(predicate);

//            var parameter = new DatabaseQueryParameter<Entity>(database.Accessor);
//            query(parameter);

//            // 额外查询条件
//            if (parameter.Predicates.Count > 0)
//            {
//                foreach (var predicate2 in parameter.Predicates)
//                {
//                    queryable = queryable.Where(predicate2);
//                }
//            }

//            // 先排序后limit
//            if (parameter.OrderFields.Count > 0)
//            {
//                foreach (var field in parameter.OrderFields)
//                {
//                    queryable = queryable.Order(field.Item1, field.Item2);
//                }
//            }
//            var total = queryable.Count();
//            if (parameter.Page > 0 && parameter.PageSize >= 0)
//            {
//                queryable = queryable.Skip((parameter.Page - 1) * parameter.PageSize).Take(parameter.PageSize);
//            }
//            var result = await queryable.ToListAsync();

//            return new ListResult<Entity>(result, total, parameter.Page, parameter.PageSize);
//        }

//        #endregion

//        #region 私有的
//        static IQueryable<T> Order<T>(this IQueryable<T> queryable, string property, bool ascending)
//        {
//            if (queryable is null || string.IsNullOrWhiteSpace(property)) return queryable;

//            var parameter = Expression.Parameter(typeof(T));

//            var body = property.Split('.').Aggregate(parameter as Expression, Expression.Property);

//            var expression = (dynamic)Expression.Lambda(body, parameter);

//            return ascending ? Queryable.OrderBy(queryable, expression) : Queryable.OrderByDescending(queryable, expression);
//        }


//        static Dictionary<Type, string[]> Includes { get; set; } = new Dictionary<Type, string[]>();
//        static IQueryable<Entity> AutoInclude<Entity>(this IQueryable<Entity> queryable, IDatabase context) where Entity : class
//        {
//            var supportNavigationProperty = context.SupportNavigationProperty;
//            if (Includes == null)
//            {
//                Includes = new Dictionary<Type, string[]>();

//            }
//            var tempQueryable = queryable;
//            if (supportNavigationProperty)
//            {
//                var propertyNames = Array.Empty<string>();
//                // 如果导航属性不存在，初始化导航属性字典
//                if (!Includes.TryGetValue(typeof(Entity), out propertyNames))
//                {
//                    foreach (var entityType in ((DbContext)context).Model.GetEntityTypes())
//                    {
//                        Includes.Add(entityType.ClrType, entityType.GetNavigations().Select(n => n.Name).ToArray());
//                    }
//                    propertyNames = Includes[typeof(Entity)];
//                }
//                foreach (var property in propertyNames)
//                {
//                    tempQueryable = tempQueryable.Include(property);
//                }
//            }
//            return tempQueryable;
//        }
//        #endregion

//    }
//}
