﻿using Core.General;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace Core.Extensions
{
    /// <summary>
    /// 枚举类型拓展
    /// </summary>
    public static class EnumExtensions
    {
        /// <summary>
        /// 获取枚举描述
        /// </summary>
        /// <param name="enum"></param>
        /// <returns></returns>
        public static string GetDescription(this Enum val)
        {
            //typeof(ApprovalStatusEnum).GetEnumOptions
            var type = val.GetType();
            var member = type.GetMember(val.ToString())[0]; //获取成员  
            var desc = member.GetCustomAttribute<DescriptionAttribute>();
            return desc?.Description ?? "未知";
        }

        /// <summary>
        /// 获取枚举选项列表
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static EnumOption[] GetEnumOptions(this Type type)
        {
            return type.GetEnumNames().Select(name =>
            {
                var member = type.GetMember(name)[0];
                var desc = member?.GetCustomAttribute<DescriptionAttribute>();
                return new EnumOption { Label = desc?.Description ?? name, Value = (int)Enum.Parse(type, name) };
            }).ToArray();
        }
    }
}
