﻿using Microsoft.AspNetCore.Http;
using SixLabors.Fonts;
using SixLabors.ImageSharp;
using SixLabors.ImageSharp.Drawing.Processing;
using SixLabors.ImageSharp.PixelFormats;
using SixLabors.ImageSharp.Processing;
using SixLaborsCaptcha.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Runtime.CompilerServices;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace Core.Extensions
{
    /// <summary>
    /// 字符串拓展
    /// </summary>
    public static class StringExtensions
    {
        /// <summary>
        /// 使用UTF8编码获取字节数组
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static byte[] GetBytes(this string str) => Encoding.UTF8.GetBytes(str);
        /// <summary>
        /// 使用UTF8编码获取字符串
        /// </summary>
        /// <param name="buffer"></param>
        /// <returns></returns>
        public static string GetString(this byte[] buffer) => Encoding.UTF8.GetString(buffer);
        /// <summary>
        /// 按密码表缩短为指定长度字符串
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <param name="digit">位数</param>
        /// <param name="cipher">密码表</param>
        /// <returns>短码</returns>
        public static string Shorten(this string str, int digit, string cipher)
        {
            var buffer = MD5.HashData(Encoding.UTF8.GetBytes(str));
            var step = (int)buffer.Length / digit;
            var builder = new StringBuilder();
            for (var i = 0; i < digit && i < buffer.Length; i++)
            {
                builder.Append(cipher[buffer[i * step] % cipher.Length]);
            }
            // 生成hash code
            return builder.ToString();
        }

        /// <summary>
        /// 按内置密码表缩短为指定长度字符串
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <param name="digit">位数</param>
        /// <returns></returns>
        public static string Shorten(this string str, int digit) => str.Shorten(digit, "dHkfcJupR2ygGO0mX5xVBWZ31KvablITMst9D4C8hjEo7iwLqeS6YQUzNrFAPn");
        /// <summary>
        /// 按内置密码表缩短为长度为6的字符串
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <returns></returns>
        public static string Shorten(this string str) => str.Shorten(6);
        
        /// <summary>
        /// 转换布尔值
        /// </summary>
        /// <param name="input">输入</param>
        /// <param name="defaultValue">默认值</param>
        /// <returns></returns>
        public static bool TryBoolean(this string input, bool defaultValue = false)
        {
            if (bool.TryParse(input, out var value)) return value;
            else return defaultValue;
        }

        /// <summary>
        /// 转换整形
        /// </summary>
        /// <param name="input">输入</param>
        /// <param name="defaultValue">默认值</param>
        /// <returns></returns>
        public static int TryInt(this string input, int defaultValue = 0)
        {
            if (int.TryParse(input, out var value)) return value;
            else return defaultValue;
        }

        /// <summary>
        /// 转换长整形
        /// </summary>
        /// <param name="input">输入</param>
        /// <param name="defaultValue">默认值</param>
        /// <returns></returns>
        public static long TryLong(this string input, long defaultValue = 0)
        {
            if (long.TryParse(input, out var value)) return value;
            else return defaultValue;
        }

        /// <summary>
        /// 转换单精度浮点型
        /// </summary>
        /// <param name="input">输入</param>
        /// <param name="defaultValue">默认值</param>
        /// <returns></returns>
        public static float TryFloat(this string input, float defaultValue = 0)
        {
            if (float.TryParse(input, out var value)) return value;
            else return defaultValue;
        }

        /// <summary>
        /// 转换双精度浮点型
        /// </summary>
        /// <param name="input">输入</param>
        /// <param name="defaultValue">默认值</param>
        /// <returns></returns>
        public static double TryDouble(this string input, double defaultValue = 0)
        {
            if (double.TryParse(input, out var value)) return value;
            else return defaultValue;
        }
    }
}
