﻿using Core;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.OpenApi.Models;
using Swashbuckle.AspNetCore.SwaggerGen;
using Swashbuckle.AspNetCore.SwaggerUI;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace Core.Extensions
{
    /// <summary>
    /// SwaggerUIExtensions
    /// </summary>
    public static class SwaggerUIExtensions
    {
        /// <summary>
        /// 生成分组文档
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="gen"></param>
        public static void SwaggerDocGroup<TEnum>(this SwaggerGenOptions gen) where TEnum : Enum
        {
            var names = Enum.GetNames(typeof(TEnum));
            if (names.Length > 0)
                foreach (var name in names)
                {
                    var member = typeof(TEnum).GetMember(name)[0];
                    var desc = member.GetCustomAttribute<DescriptionAttribute>()?.Description;
                    gen.SwaggerDoc(name, new OpenApiInfo()
                    {
                        Title = string.IsNullOrEmpty(desc) ? name : desc
                    });
                }


        }
        /// <summary>
        /// 使用SwaggerUI，并通过特定类型分组
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="app"></param>
        /// <returns></returns>
        public static IApplicationBuilder UseSwaggerUIGroup<TEnum>(this IApplicationBuilder app) where TEnum : Enum
        {
            var names = Enum.GetNames(typeof(TEnum));
            if (names.Length > 0)
                app.UseSwaggerUI(c =>
                {
                    foreach (var name in names)
                    {
                        var member = typeof(TEnum).GetMember(name)[0];
                        var desc = member.GetCustomAttribute<DescriptionAttribute>()?.Description;
                        c.SwaggerEndpoint($"/swagger/{name}/swagger.json", string.IsNullOrEmpty(desc) ? name : desc);
                    }
                });
            return app;
        }
    }
}
