﻿using Core.Extensions;
using SixLaborsCaptcha.Core;
using System;
using System.Collections.Generic;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Core
{
    /// <summary>
    /// 工具库
    /// </summary>
    public static class Utils
    {
        /// <summary>
        /// 获取主程序引用的程序集
        /// </summary>
        /// <returns></returns>
        public static Assembly[] GetReferencedAssemblies()
        {
            var names = Assembly.GetEntryAssembly().GetReferencedAssemblies();
            var list = new List<Assembly>();

            foreach (var name in names)
            {
                list.Add(Assembly.Load(name));
            }
            return list.ToArray();
        }

        /// <summary>
        /// 获取实现了 IEntity 接口的实体类型
        /// </summary>
        /// <param name="assembly"></param>
        /// <returns></returns>
        public static Type[] GetEntityTypes()
        {
            var types = new List<Type>();
            foreach (var assembly in GetReferencedAssemblies())
            {
                var entityType = typeof(IEntity);
                var types2 = assembly.GetTypes().Where(p => p.IsPublic && p.IsClass && p.IsAssignableTo(entityType) && !p.IsAbstract).ToList();
                types.AddRange(types2);
            }
            return types.ToArray();
        }
        /// <summary>
        /// 获取实现了 IEntity 接口的实体类型
        /// </summary>
        /// <returns></returns>
        public static Type[] GetEntitySeedDataTypes()
        {
            var types = new List<Type>();
            foreach (var assembly in GetReferencedAssemblies())
            {
                var entityType = typeof(ISqlSugarEntitySeedData<>);
                var types2 = assembly.GetTypes()
                    .Where(p => p.IsPublic && p.IsClass &&
                                p.GetInterfaces().Any(i => i.IsGenericType &&
                                i.GetGenericTypeDefinition() == entityType) &&
                                !p.IsAbstract).ToList();
                types.AddRange(types2);
            }
            return types.ToArray();
        }



        /// <summary>
        /// 随机字符串
        /// </summary>
        /// <param name="length">字符串长度</param>
        /// <returns></returns>
        public static string RandomString(int length = 6)
        {
            const string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"; // 定义生成随机字符串的字符范围

            Random random = new Random();
            StringBuilder stringBuilder = new StringBuilder(length);
            for (int i = 0; i < length; i++)
            {
                int index = random.Next(chars.Length);
                stringBuilder.Append(chars[index]);
            }

            return stringBuilder.ToString();
        }
        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="length">验证码长度</param>
        /// <returns></returns>
        public static byte[] Captcha(int length = 6)
        {
            var options = new SixLaborsCaptchaOptions();
            return Captcha(options, length);
        }

        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="length">验证码长度</param>
        /// <param name="captchaOptions"></param>
        /// <returns></returns>
        public static byte[] Captcha(SixLaborsCaptchaOptions captchaOptions, int length = 6)
        {
            var randomString = RandomString(length);
            var module = new SixLaborsCaptchaModule(captchaOptions);
            return module.Generate(randomString);
        }

        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="code">验证码内容</param>
        /// <returns></returns>
        public static byte[] Captcha(string code)
        {
            var options = new SixLaborsCaptchaOptions();
            return Captcha(code, options);
        }

        /// <summary>
        /// 生成验证码
        /// </summary>
        /// <param name="code">验证码内容</param>
        /// <param name="captchaOptions"></param>
        /// <returns></returns>
        public static byte[] Captcha(string code, SixLaborsCaptchaOptions captchaOptions)
        {
            var module = new SixLaborsCaptchaModule(captchaOptions);
            return module.Generate(code);
        }


    }
}
